@@ -17,7 +17,7 @@ import sys
 import warnings
 from functools import wraps
 
-from typing import Any, Optional
+from typing import Optional
 
 import click
 import numpy
@@ -28,7 +28,13 @@ from sdsstools.daemonizer import DaemonGroup
 from jaeger import can_log, config, log
 from jaeger.commands.bootloader import load_firmware
 from jaeger.commands.calibration import calibrate_positioner
-from jaeger.exceptions import JaegerError, JaegerUserWarning
+from jaeger.commands.goto import goto as goto_
+from jaeger.exceptions import (
+    FPSLockedError,
+    JaegerError,
+    JaegerUserWarning,
+    TrajectoryError,
+)
 from jaeger.fps import FPS, LOCK_FILE
 from jaeger.positioner import Positioner
 from jaeger.testing import VirtualFPS
@@ -73,7 +79,6 @@ class FPSWrapper(object):
         initialise=True,
         npositioners=10,
         enable_low_temperature=True,
-        use_lock=True,
     ):
 
         self.profile = profile
@@ -83,7 +88,6 @@ class FPSWrapper(object):
         self.ieb = ieb
         self.enable_low_temperature = enable_low_temperature
         self.initialise = initialise
-        self.use_lock = use_lock
 
         self.vpositioners = []
         self.npositioners = npositioners
@@ -107,8 +111,7 @@ class FPSWrapper(object):
 
         if self.initialise:
             await self.fps.initialise(
-                enable_low_temperature=self.enable_low_temperature,
-                use_lock=self.use_lock,
+                enable_low_temperature=self.enable_low_temperature
             )
             if self.enable_low_temperature is False:
                 warnings.warn(
@@ -222,6 +225,9 @@ def jaeger(
     if config_file:
         config.load(config_file)
 
+    if no_lock:
+        config["fps"]["use_lock"] = False
+
     actor_config = config.get("actor", {})
 
     if verbose == 1:
@@ -258,7 +264,6 @@ def jaeger(
         ieb=ieb,
         npositioners=npositioners,
         enable_low_temperature=enable_low_temperature,
-        use_lock=not no_lock,
     )
 
 
@@ -281,7 +286,7 @@ LOG_FILE = os.path.join(
 )
 @pass_fps
 @cli_coro
-async def actor(fps_maker, no_tron):
+async def actor(fps_maker, no_tron: bool = False):
     """Runs the actor."""
 
     try:
@@ -295,12 +300,15 @@ async def actor(fps_maker, no_tron):
     if no_tron:
         actor_config.pop("tron", None)
 
+    # Do not initialise FPS so that we can define the actor instance first.
+    fps_maker.initialise = False
+
     async with fps_maker as fps:
-        actor_: Any = await JaegerActor.from_config(actor_config, fps).start()
-        await actor_.start_status_server(
-            config["actor"]["status"]["port"],
-            delay=config["actor"]["status"]["delay"],
-        )
+        actor_: JaegerActor = JaegerActor.from_config(actor_config, fps)
+
+        await fps.initialise()
+
+        await actor_.start()
         await actor_.run_forever()
 
 
@@ -457,42 +465,90 @@ async def calibrate(fps_maker, positioner_id, motors, datums, cogging):
 
 
 @jaeger.command()
-@click.argument("positioner_id", metavar="POSITIONER", type=int)
-@click.argument("alpha", metavar="ALPHA", type=float)
-@click.argument("beta", metavar="BETA", type=float)
+@click.argument("POSITIONER-IDS", type=int, nargs=-1)
+@click.argument("ALPHA", type=click.FloatRange(-10.0, 370.0))
+@click.argument("BETA", type=click.FloatRange(-10.0, 370.0))
 @click.option(
+    "-r",
+    "--relative",
+    is_flag=True,
+    help="Whether this is a relative move",
+)
+@click.option(
+    "-s",
     "--speed",
-    type=(float, float),
-    default=(None, None),
-    help="The speed for the alpha and beta motors.",
-    show_default=True,
+    type=click.FloatRange(100.0, 4000.0),
+    help="The speed for both alpha and beta arms, in RPS on the input.",
+)
+@click.option(
+    "-a",
+    "--all",
+    is_flag=True,
+    default=False,
+    help="Applies to all valid positioners.",
+)
+@click.option(
+    "-f",
+    "--force",
+    is_flag=True,
+    default=False,
+    help="Forces a move to happen.",
+)
+@click.option(
+    "--go-cowboy",
+    is_flag=True,
+    help="If set, does not use kaiju-validated trajectories.",
+)
+@click.option(
+    "--use-sync/-no-use-sync",
+    " /-S",
+    default=True,
+    help="Whether to use the SYNC line to start the trajectory.",
 )
 @pass_fps
 @cli_coro
-async def goto(fps_maker, positioner_id, alpha, beta, speed=None):
-    """Moves a robot to a given position."""
+async def goto(
+    fps_maker,
+    positioner_ids: tuple[int, ...] | list[int],
+    alpha: float,
+    beta: float,
+    speed: float | None,
+    all: bool = False,
+    force: bool = False,
+    relative: bool = False,
+    use_sync: bool = True,
+    go_cowboy: bool = False,
+):
+    """Sends positioners to a given (alpha, beta) position."""
 
-    if alpha < 0 or alpha >= 360:
-        raise click.UsageError("alpha must be in the range [0, 360)")
+    with fps_maker as fps:
 
-    if beta < 0 or beta >= 360:
-        raise click.UsageError("beta must be in the range [0, 360)")
+        if all:
+            if not force:
+                raise JaegerError("Use --force to move all positioners at once.")
+            positioner_ids = list(fps.positioners.keys())
+        else:
+            positioner_ids = list(positioner_ids)
 
-    if speed[0] or speed[1]:
-        if speed[0] < 0 or speed[0] >= 3000 or speed[1] < 0 or speed[1] >= 3000:
-            raise click.UsageError("speed must be in the range [0, 3000)")
+        if not relative:
+            if alpha < 0 or beta < 0:
+                raise JaegerError("Negative angles only allowed in relative mode.")
 
-    async with fps_maker as fps:
+        new_positions = {}
+        for pid in positioner_ids:
+            new_positions[pid] = (alpha, beta)
 
-        positioner = fps.positioners[positioner_id]
-        result = await positioner.initialise()
-        if not result:
-            log.error("positioner is not connected or failed to initialise.")
-            return
-
-        await positioner.goto(alpha=alpha, beta=beta, speed=(speed[0], speed[1]))
-
-    return
+        try:
+            await goto_(
+                fps,
+                new_positions,
+                speed=speed,
+                relative=relative,
+                use_sync_line=use_sync,
+                go_cowboy=go_cowboy,
+            )
+        except (JaegerError, TrajectoryError) as err:
+            raise JaegerError(f"Goto command failed: {err}")
 
 
 @jaeger.command(name="set-positions")
@@ -694,6 +750,9 @@ async def unwind(fps_maker, collision_buffer: float | None = None, force: bool =
 
     async with fps_maker as fps:
 
+        if fps.locked:
+            FPSLockedError("The FPS is locked.")
+
         await fps.update_position()
         positions = {p.positioner_id: (p.alpha, p.beta) for p in fps.values()}
 
@@ -701,6 +760,7 @@ async def unwind(fps_maker, collision_buffer: float | None = None, force: bool =
         trajectory = await unwind(
             positions,
             collision_buffer=collision_buffer,
+            disabled=[pid for pid in fps.positioners if fps.positioners[pid].disabled],
             force=force,
         )
 
@@ -712,20 +772,33 @@ async def unwind(fps_maker, collision_buffer: float | None = None, force: bool =
 
 @jaeger.command()
 @click.argument("EXPLODE-DEG", type=float)
+@click.option("--one", type=int, help="Only explode this positioner.")
 @pass_fps
 @cli_coro
-async def explode(fps_maker, explode_deg: float):
+async def explode(
+    fps_maker,
+    explode_deg: float,
+    one: int | None = None,
+):
     """Explodes the array."""
 
     from jaeger.kaiju import explode
 
     async with fps_maker as fps:
 
+        if fps.locked:
+            FPSLockedError("The FPS is locked.")
+
         await fps.update_position()
         positions = {p.positioner_id: (p.alpha, p.beta) for p in fps.values()}
 
         log.info("Calculating trajectory.")
-        trajectory = await explode(positions, explode_deg=explode_deg)
+        trajectory = await explode(
+            positions,
+            explode_deg=explode_deg,
+            disabled=[pid for pid in fps.positioners if fps.positioners[pid].disabled],
+            positioner_id=one,
+        )
 
         log.info("Executing explode trajectory.")
         await fps.send_trajectory(trajectory)
