@@ -13,6 +13,7 @@ import json
 import logging
 import os
 from tempfile import NamedTemporaryFile
+from time import time
 
 import clu
 import clu.protocol
@@ -20,7 +21,7 @@ from clu.tools import ActorHandler
 
 import jaeger
 from jaeger import FPS, __version__, log
-from jaeger.exceptions import JaegerUserWarning
+from jaeger.exceptions import JaegerError, JaegerUserWarning
 
 
 __all__ = ["JaegerActor"]
@@ -51,7 +52,14 @@ def merge_json(base: str, custom: str | None, write_temporary_file=False):
 class JaegerActor(clu.LegacyActor):
     """The jaeger SDSS-style actor."""
 
-    def __init__(self, fps: FPS, *args, ieb_status_delay=60, **kwargs):
+    def __init__(
+        self,
+        fps: FPS,
+        *args,
+        ieb_status_delay: float = 60.0,
+        observatory: str | None = None,
+        **kwargs,
+    ):
 
         jaeger.actor_instance = self
 
@@ -72,6 +80,15 @@ class JaegerActor(clu.LegacyActor):
         # command (the first argument is always the actor command).
         self.parser_args = [fps]
 
+        # Set the observatory where the actor is running.
+        if observatory is None:
+            try:
+                self.observatory = os.environ["OBSERVATORY"]
+            except KeyError:
+                raise JaegerError("Observatory not passed and $OBSERVATORY is not set.")
+        else:
+            self.observatory = observatory
+
         super().__init__(*args, **kwargs)
 
         self.version = __version__
@@ -86,7 +103,11 @@ class JaegerActor(clu.LegacyActor):
         if log.warnings_logger:
             log.warnings_logger.addHandler(self.actor_handler)
 
-        self.__status_watcher_task = asyncio.create_task(self._status_watcher())
+        self._alive_task = asyncio.create_task(self._report_alive())
+        self._status_watcher_task = asyncio.create_task(self._status_watcher())
+
+        self.fps.alerts.set_actor(self)
+        self.fps.chiller.set_actor(self)
 
     async def start_status_server(self, port, delay=1):
         """Starts a server that outputs the status as a JSON on a timer."""
@@ -102,6 +123,13 @@ class JaegerActor(clu.LegacyActor):
 
         self.log.info(f"starting status server on {self.host}:{port}")
 
+    async def _report_alive(self):
+        """Outputs the ``alive_at`` keyword."""
+
+        while True:
+            self.write("d", {"alive_at": time()}, broadcast=True)
+            await asyncio.sleep(60)
+
     async def _report_status_cb(self, transport):
         """Reports the status to the status server."""
 
