@@ -9,21 +9,24 @@
 from __future__ import annotations
 
 import asyncio
+import json
 import pathlib
+from time import time
 
-from typing import TYPE_CHECKING, List, Tuple
+from typing import TYPE_CHECKING
 
 import click
 import numpy
 
 import clu
 
-from jaeger import config
+from jaeger.can import JaegerCAN
 from jaeger.commands import CommandID, SetCurrent, Trajectory
+from jaeger.commands.goto import goto as goto_
+from jaeger.commands.trajectory import send_trajectory
 from jaeger.exceptions import JaegerError, TrajectoryError
-from jaeger.utils import get_goto_move_time
 
-from . import jaeger_parser
+from . import JaegerCommandType, jaeger_parser
 
 
 if TYPE_CHECKING:
@@ -43,6 +46,7 @@ __all__ = [
     "unlock",
     "current",
     "status",
+    "set_collision_margin",
 ]
 
 
@@ -63,8 +67,15 @@ def check_positioners(positioner_ids, command, fps, initialised=False):
 
 @jaeger_parser.command()
 @click.argument("POSITIONER-IDS", type=int, nargs=-1)
-@click.argument("ALPHA", type=click.FloatRange(-360.0, 360.0))
-@click.argument("BETA", type=click.FloatRange(-360.0, 360.0))
+@click.argument("ALPHA", type=click.FloatRange(-10.0, 370.0), required=False)
+@click.argument("BETA", type=click.FloatRange(-10.0, 370.0), required=False)
+@click.option(
+    "-l",
+    "--from-file",
+    type=click.Path(exists=True, dir_okay=False),
+    help="Loads the trajectory from a JSON trajectory file. Unless --go-cowboy is used "
+    "only the end points of the trajectories for each positioner are used.",
+)
 @click.option(
     "-r",
     "--relative",
@@ -75,8 +86,7 @@ def check_positioners(positioner_ids, command, fps, initialised=False):
     "-s",
     "--speed",
     type=click.FloatRange(100.0, 4000.0),
-    nargs=2,
-    help="The speed of both alpha and beta arms, in RPS on the input.",
+    help="The speed for both alpha and beta arms, in RPS on the input.",
 )
 @click.option(
     "-a",
@@ -95,88 +105,100 @@ def check_positioners(positioner_ids, command, fps, initialised=False):
 @click.option(
     "--use-sync/-no-use-sync",
     " /-S",
-    default=True,
+    default=None,
     help="Whether to use the SYNC line to start the trajectory.",
 )
+@click.option(
+    "--go-cowboy",
+    is_flag=True,
+    help="If set, does not use kaiju-validated trajectories.",
+)
 async def goto(
     command: Command[JaegerActor],
     fps: FPS,
-    positioner_ids: Tuple[int, ...],
-    alpha: float,
-    beta: float,
-    speed: Tuple[float, float],
+    positioner_ids: tuple[int, ...] | list[int],
+    alpha: float | None,
+    beta: float | None,
+    from_file: str | None,
+    speed: float | None,
     all: bool = False,
     force: bool = False,
     relative: bool = False,
     use_sync: bool = True,
+    go_cowboy: bool = False,
 ):
     """Sends positioners to a given (alpha, beta) position."""
 
     assert command.actor
 
-    if all:
-        if not force:
-            return command.fail(error="Use --force to move all positioners at once.")
-        pids: List[int] = list(fps.positioners.keys())
-    else:
-        pids = list(positioner_ids)
+    if from_file is None and (alpha is None or beta is None):
+        return command.fail(error="alpha and beta or --from-file are required.")
 
-    if not relative:
-        if alpha < 0 or beta < 0:
-            return command.fail(error="Negative angles only allowed in relative mode.")
-
-    if not check_positioners(pids, command, fps, initialised=True):
-        return
+    if fps.locked:
+        return command.fail(error="FPS is locked. Cannot send goto.")
 
     if fps.moving:
         return command.fail(error="FPS is moving. Cannot send goto.")
 
-    if speed is None:
-        default_speed = config["positioner"]["motor_speed"]
-        speed = (default_speed, default_speed)
+    if from_file is not None:
+        trajectory = json.loads(open(from_file, "r").read())["trajectory"]
 
-    max_time = 0.0
+        if go_cowboy is True:
+            try:
+                await send_trajectory(
+                    fps,
+                    trajectory,
+                    use_sync_line=use_sync,
+                    command=command,
+                    extra_dump_data={"kaiju_trajectory": False},
+                )
+                return command.finish()
+            except (JaegerError, TrajectoryError) as err:
+                return command.fail(error=f"Goto command failed: {err}")
 
-    for pid in pids:
+        else:
+            new_positions = {}
+            for positioner_id in trajectory:
+                alpha = trajectory[positioner_id]["alpha"][-1][0]
+                beta = trajectory[positioner_id]["beta"][-1][0]
+                new_positions[int(positioner_id)] = (alpha, beta)
 
-        # Manually calculate the max move time we'll encounter.
-        p_alpha, p_beta = fps[pid].position
+    else:
 
-        if p_alpha is None or p_beta is None:
-            return command.fail(error="Some positioners do not know their positions.")
+        assert alpha is not None and beta is not None
 
-        delta_alpha = abs(p_alpha - alpha) if not relative else alpha
-        delta_beta = abs(p_beta - beta) if not relative else beta
+        if all:
+            if not force:
+                return command.fail("Use --force to move all positioners at once.")
+            positioner_ids = list(fps.positioners.keys())
+        else:
+            positioner_ids = list(positioner_ids)
 
-        time_alpha = get_goto_move_time(
-            delta_alpha,
-            speed=speed[0] or fps[pid].speed[0],
-        )
-        time_beta = get_goto_move_time(
-            delta_beta,
-            speed=speed[1] or fps[pid].speed[1],
-        )
+        if not relative:
+            if alpha < 0 or beta < 0:
+                return command.fail("Negative angles only allowed in relative mode.")
 
-        if time_alpha > max_time:
-            max_time = time_alpha
-        if time_beta > max_time:
-            max_time = time_beta
+        if not check_positioners(positioner_ids, command, fps, initialised=True):
+            return
 
-    command.info(move_time=round(max_time, 2))
+        new_positions = {}
+        for pid in positioner_ids:
+            new_positions[pid] = (alpha, beta)
 
     try:
-        await command.actor.fps.goto(
-            pids,
-            alpha,
-            beta,
+        await goto_(
+            fps,
+            new_positions,
             speed=speed,
             relative=relative,
             use_sync_line=use_sync,
+            command=command,
+            go_cowboy=go_cowboy,
         )
-    except JaegerError as err:
+    except (JaegerError, TrajectoryError) as err:
         return command.fail(error=f"Goto command failed: {err}")
 
-    command.finish(text="Destination reached")
+    command.finish()
 
 
 @jaeger_parser.command()
@@ -242,22 +264,23 @@ async def initialise(command, fps, positioner_id, datums=False):
 
 
 @jaeger_parser.command()
-@click.argument("POSITIONER-IDS", type=int, nargs=-1, required=False)
-@click.option(
-    "-f",
-    "--full",
-    is_flag=True,
-    default=False,
-    help="outputs more statuses.",
-)
-async def status(command, fps, positioner_ids, full):
+@click.argument("POSITIONERS", type=int, nargs=-1, required=False)
+async def status(command: JaegerCommandType, fps: FPS, positioners):
     """Reports the position and status bit of a list of positioners."""
 
-    positioner_ids = positioner_ids or list(fps.positioners.keys())
+    positioner_ids = positioners or list(fps.positioners.keys())
 
     if not check_positioners(positioner_ids, command, fps):
         return
 
+    if len(positioners) == 0:
+        command.actor.write("d", {"alive_at": time()}, broadcast=True)
+        command.info(locked=fps.locked)
+        command.info(folded=(await fps.is_folded()))
+        command.info(n_positioners=len(fps.positioners))
+        command.info(fps_status=f"0x{fps.status.value:x}")
+        command.info(message={k: int(v) for k, v in fps.alerts.keywords.items()})
+
     try:
         await fps.update_status(positioner_ids=0)
         await fps.update_position(positioner_ids=positioner_ids)
@@ -271,10 +294,6 @@ async def status(command, fps, positioner_ids, full):
         )
     ).get_replies()
 
-    command.info(locked=fps.locked)
-
-    command.info(n_positioners=len(fps.positioners))
-
     for pid in sorted(positioner_ids):
         p = fps[pid]
 
@@ -283,7 +302,7 @@ async def status(command, fps, positioner_ids, full):
 
         n_trajs_pid = n_trajs[pid] if n_trajs[pid] is not None else "?"
 
-        if pid in fps.positioner_to_bus:
+        if pid in fps.positioner_to_bus and isinstance(fps.can, JaegerCAN):
             interface, bus = fps.positioner_to_bus[pid]
             interface = fps.can.interfaces.index(interface) + 1
         else:
@@ -298,6 +317,8 @@ async def status(command, fps, positioner_ids, full):
                 beta_pos,
                 f"0x{int(p.status):x}",
                 p.initialised,
+                p.disabled,
+                p.offline,
                 p.is_bootloader() or False,
                 p.firmware or "?",
                 interface,
@@ -306,9 +327,7 @@ async def status(command, fps, positioner_ids, full):
             ],
         )
 
-    command.info(fps_status=f"0x{fps.status.value:x}")
-
-    if full:
+    if len(positioners) == 0:
         await clu.Command("ieb status", parent=command).parse()
 
     command.set_status(clu.CommandStatus.DONE)
@@ -510,3 +529,37 @@ async def reload(command, fps):
         return command.fail(error=f"Initialisation failed: {err}")
 
     return command.finish(text="FPS was reinitialised.")
+
+
+@jaeger_parser.command(name="set-collision-margin")
+@click.argument("MARGIN", type=click.IntRange(-30, 30))
+@click.option(
+    "-p",
+    "--positioners",
+    type=str,
+    help="Comma-separated positioners to which to apply the margin. "
+    "If not set, applies to all the positioners.",
+)
+async def set_collision_margin(
+    command: JaegerCommandType,
+    fps: FPS,
+    margin: int,
+    positioners: str | None = None,
+):
+    """Change the collision margin. The collision margin must be -30 to 30 degrees."""
+
+    if positioners is not None:
+        positioner_ids = list(map(int, positioners.split(",")))
+    else:
+        positioner_ids = None
+
+    margin_command = await fps.send_command(
+        "SET_INCREASE_COLLISION_MARGIN",
+        positioner_ids=positioner_ids,
+        margin=margin,
+    )
+
+    if margin_command.status.failed:
+        return command.fail("Failed updating collision margin.")
+
+    return command.finish()
