@@ -35,7 +35,7 @@ class CommandID(enum.IntEnum, metaclass=TypesEnumMeta):
     SEND_NEW_TRAJECTORY = 10
     SEND_TRAJECTORY_DATA = 11
     TRAJECTORY_DATA_END = 12
-    TRAJECTORY_TRANSMISSION_ABORT = 13
+    SEND_TRAJECTORY_ABORT = 13
     START_TRAJECTORY = 14
     STOP_TRAJECTORY = 15
     COLLISION_DETECTED = 18
