@@ -9,13 +9,17 @@
 from __future__ import annotations
 
 import asyncio
+import json
+import os
 import pathlib
 import time
 import warnings
+from glob import glob
 
-from typing import TYPE_CHECKING, Dict, List, Tuple, cast
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, cast
 
 import numpy
+from astropy.time import Time
 
 import drift
 from sdsstools import read_yaml_file
@@ -24,12 +28,15 @@ from jaeger import config, log
 from jaeger.commands import Command, CommandID
 from jaeger.exceptions import FPSLockedError, JaegerUserWarning, TrajectoryError
 from jaeger.ieb import IEB
-from jaeger.maskbits import FPSStatus, PositionerStatus, ResponseCode
+from jaeger.maskbits import FPSStatus, ResponseCode
 from jaeger.utils import int_to_bytes
 
 
 if TYPE_CHECKING:
+    from clu.command import Command as CluCommand
+
     from jaeger import FPS
+    from jaeger.actor import JaegerActor
 
 
 __all__ = [
@@ -55,8 +62,11 @@ async def send_trajectory(
     fps: FPS,
     trajectories: str | pathlib.Path | TrajectoryDataType,
     use_sync_line: bool | None = None,
-    send_trajectory=True,
-    start_trajectory=True,
+    send_trajectory: bool = True,
+    start_trajectory: bool = True,
+    command: Optional[CluCommand[JaegerActor]] = None,
+    dump: bool | str = True,
+    extra_dump_data: dict[str, Any] = {},
 ) -> Trajectory:
     """Sends a set of trajectories to the positioners.
 
@@ -87,6 +97,16 @@ async def send_trajectory(
         If `True`, runs the trajectory after sending it. Otherwise, returns
         the `.Trajectory` instance after sending the data. Ignored if
         `send_trajectory=False`.
+    command
+        A command to which to output informational messages.
+    dump
+        Whether to dump a JSON with the trajectory to disk. If `True`,
+        the trajectory is stored in ``/data/logs/jaeger/trajectories/<MJD>`` with
+        a unique sequence for each trajectory. A string can be passed with a
+        custom path. The dump file is created only if `.Trajectory.start` is
+        called, regardless of whether the trajectory succeeds.
+    extra_dump_data
+        A dictionary with additional parameters to add to the dump JSON.
 
     Raises
     ------
@@ -108,7 +128,12 @@ async def send_trajectory(
 
     """
 
-    traj = Trajectory(fps, trajectories)
+    traj = Trajectory(
+        fps,
+        trajectories,
+        dump=dump,
+        extra_dump_data=extra_dump_data,
+    )
 
     if use_sync_line is None:
         use_sync_line = config["fps"]["use_sync_line"]
@@ -122,7 +147,10 @@ async def send_trajectory(
     if send_trajectory is False:
         return traj
 
-    log.debug("sending trajectory data.")
+    msg = "Sending trajectory data."
+    log.debug(msg)
+    if command:
+        command.debug(msg)
 
     try:
         await traj.send()
@@ -131,14 +159,29 @@ async def send_trajectory(
             f"Something went wrong sending the trajectory: {err}",
             err.trajectory,
         )
+    finally:
+        if traj.dump_file:
+            traj.dump_trajectory()
+            if command:
+                command.debug(trajectory_dump_file=traj.dump_file)
+
+    msg = f"Trajectory sent in {traj.data_send_time:.1f} seconds."
+    log.debug(msg)
+    if command:
+        command.debug(msg)
 
-    log.debug(f"Trajectory sent in {traj.data_send_time:1f} seconds.")
     log.info(f"Expected time to complete trajectory: {traj.move_time:.2f} seconds.")
+    if command and traj.move_time:
+        command.info(move_time=round(traj.move_time, 2))
 
     if start_trajectory is False:
         return traj
 
-    log.info("starting trajectory ...")
+    msg = "Starting trajectory ..."
+    log.info(msg)
+    if command:
+        command.info(msg)
+
     try:
         await traj.start(use_sync_line=use_sync_line)
     except TrajectoryError as err:
@@ -153,8 +196,17 @@ async def send_trajectory(
             f"Something went wrong during the trajectory: {err}.{elapsed_msg}",
             err.trajectory,
         )
+    finally:
+        if traj.dump_file and command:
+            command.debug(trajectory_dump_file=traj.dump_file)
 
-    log.info("All positioners have reached their destinations.")
+    if command:
+        command.info(folded=(await fps.is_folded()))
+
+    msg = "All positioners have reached their destinations."
+    log.info(msg)
+    if command:
+        command.info(msg)
 
     return traj
 
@@ -167,15 +219,23 @@ class Trajectory(object):
 
     Parameters
     ----------
-    fps : .FPS
+    fps
         The instance of `.FPS` that will receive the trajectory.
-    trajectories : `str` or `dict`
+    trajectories
         Either a path to a YAML file to read or a dictionary with the
         trajectories. In either case the format must be a dictionary in
         which the keys are the ``positioner_ids`` and each value is a
         dictionary containing two keys: ``alpha`` and ``beta``, each
         pointing to a list of tuples ``(position, time)``, where
         ``position`` is in degrees and ``time`` is in seconds.
+    dump
+        Whether to dump a JSON with the trajectory to disk. If `True`,
+        the trajectory is stored in ``/data/logs/jaeger/trajectories/<MJD>`` with
+        a unique sequence for each trajectory. A string can be passed with a
+        custom path. The dump file is created only if `.Trajectory.start` is
+        called, regardless of whether the trajectory succeeds.
+    extra_dump_data
+        A dictionary with additional parameters to add to the dump JSON.
 
     Raises
     ------
@@ -203,6 +263,8 @@ class Trajectory(object):
         self,
         fps: FPS,
         trajectories: str | pathlib.Path | TrajectoryDataType,
+        dump: bool | str = True,
+        extra_dump_data: dict[str, Any] = {},
     ):
 
         self.fps = fps
@@ -220,7 +282,11 @@ class Trajectory(object):
             )
 
         if isinstance(trajectories, (str, pathlib.Path)):
-            self.trajectories = cast(TrajectoryDataType, read_yaml_file(trajectories))
+            path = pathlib.Path(trajectories)
+            if path.suffix == ".json":
+                self.trajectories = json.loads(open(path, "r").read())["trajectory"]
+            else:
+                self.trajectories = cast(dict, read_yaml_file(trajectories))
         elif isinstance(trajectories, dict):
             self.trajectories = trajectories
         else:
@@ -250,8 +316,39 @@ class Trajectory(object):
         self.start_time: float | None = None
         self.end_time: float | None = None
 
+        self.use_sync_line: bool = True
         self._ready_to_start = False
 
+        self.dump_data = {
+            "start_time": time.time(),
+            "success": False,
+            "trajectory_send_time": None,
+            "trajectory_start_time": None,
+            "end_time": None,
+            "use_sync_line": True,
+            "extra": extra_dump_data,
+            "trajectory": self.trajectories,
+            "initial_positions": self.fps.get_positions_dict(),
+            "final_positions": {},
+        }
+
+        if dump is False:
+            self.dump_file = None
+        elif isinstance(dump, (str, pathlib.Path)):
+            self.dump_file = str(dump)
+        else:
+            dirname = config["positioner"]["trajectory_dump_path"]
+            mjd = str(int(Time.now().mjd))
+            dirname = os.path.join(dirname, mjd)
+
+            files = list(sorted(glob(os.path.join(dirname, "*.json"))))
+            if len(files) == 0:
+                seq = 1
+            else:
+                seq = int(files[-1].split("-")[-1].split(".")[0]) + 1
+
+            self.dump_file = os.path.join(dirname, f"trajectory-{mjd}-{seq:04d}.json")
+
     def validate(self):
         """Validates the trajectory."""
 
@@ -274,12 +371,6 @@ class Trajectory(object):
             for arm in ["alpha", "beta"]:
                 data = numpy.array(list(zip(*trajectory[arm]))[0])
 
-                # if numpy.any(data > 360) or numpy.any(data < 0):
-                #     raise TrajectoryError(
-                #         f"Positioner {pid}: trajectory has points out of range.",
-                #         self,
-                #     )
-
                 if arm == "beta":
                     if config.get("safe_mode", False):
                         if config["safe_mode"] is True:
@@ -300,6 +391,7 @@ class Trajectory(object):
         self.move_time = 0.0
 
         await self.fps.stop_trajectory()
+        await self.fps.stop_trajectory(clear_flags=True)
 
         if not await self.fps.update_status(
             positioner_ids=list(self.trajectories),
@@ -308,8 +400,6 @@ class Trajectory(object):
             self.failed = True
             raise TrajectoryError("Some positioners did not respond.", self)
 
-        log.debug("Sending data points:")
-
         # Check that all positioners are ready to receive a new trajectory.
         for pos_id in self.trajectories:
 
@@ -317,9 +407,8 @@ class Trajectory(object):
             status = positioner.status
 
             if positioner.disabled:
-                self.failed = True
                 raise TrajectoryError(
-                    f"positioner_id={pos_id} is disabled but was "
+                    f"positioner_id={pos_id} is disabled/offline but was "
                     "included in the trajectory.",
                     self,
                 )
@@ -338,8 +427,6 @@ class Trajectory(object):
 
             traj_pos = self.trajectories[pos_id]
 
-            log.debug(f"{pos_id}: {traj_pos}")
-
             self.n_points[pos_id] = (len(traj_pos["alpha"]), len(traj_pos["beta"]))
 
             # Gets maximum time for this positioner
@@ -457,6 +544,8 @@ class Trajectory(object):
         if self.move_time is None:
             raise TrajectoryError("move_time not set.", self)
 
+        self.use_sync_line = use_sync_line
+
         if use_sync_line:
 
             if not isinstance(self.fps.ieb, IEB) or self.fps.ieb.disabled:
@@ -497,51 +586,51 @@ class Trajectory(object):
 
         self.start_time = time.time()
 
-        min_trajectory_time = 2.0
-        PS = PositionerStatus
+        # min_trajectory_time = 5.0
+        # PS = PositionerStatus
 
         try:
 
-            # The positioners take a bit to report that they are moving so if the
-            # move time is too short, we don't try to check if the positioners started
-            # moving (but we'll check later that they arrived to their positions.)
-            if self.move_time >= min_trajectory_time:
-                await asyncio.sleep(min_trajectory_time)
-                await self.fps.update_status()
+            # # The positioners take a bit to report that they are moving so if the
+            # # move time is too short, we don't try to check if the positioners started
+            # # moving (but we'll check later that they arrived to their positions.)
+            # if self.move_time >= min_trajectory_time:
+            #     await asyncio.sleep(min_trajectory_time)
+            #     await self.fps.update_status()
 
-                if self.fps.status & FPSStatus.IDLE:
-                    raise TrajectoryError("Move failed to start.")
+            #     if self.fps.status & FPSStatus.IDLE:
+            #         raise TrajectoryError("Move failed to start.")
 
-                sbits = numpy.array([p.status for p in self.fps.values()])
-                not_moving = numpy.where(sbits & PS.DISPLACEMENT_COMPLETED)
-                if not_moving[0].any():
-                    not_moving_pids = numpy.array(list(self.fps))[not_moving[0]]
+            #     sbits = numpy.array([p.status for p in self.fps.values()])
+            #     not_moving = numpy.where(sbits & PS.DISPLACEMENT_COMPLETED)
+            #     if not_moving[0].any():
+            #         not_moving_pids = numpy.array(list(self.fps))[not_moving[0]]
 
-                    # Before reporting that the positioner is not moving, check if it's
-                    # already there.
-                    await self.fps.update_position()
-                    positions = self.fps.get_positions()
+            #         # Before reporting that the positioner is not moving, check if
+            #         # it's already there.
+            #         await self.fps.update_position()
+            #         positions = self.fps.get_positions()
 
-                    really_not_moving = []
-                    for pid in not_moving_pids:
-                        if pid not in self.trajectories:
-                            continue
+            #         really_not_moving = []
+            #         for pid in not_moving_pids:
+            #             if pid not in self.trajectories:
+            #                 continue
 
-                        current = positions[positions[:, 0] == pid][0, 1:]
-                        alpha = self.trajectories[pid]["alpha"][-1][0]
-                        beta = self.trajectories[pid]["beta"][-1][0]
+            #             current = positions[positions[:, 0] == pid][0, 1:]
+            #             alpha = self.trajectories[pid]["alpha"][-1][0]
+            #             beta = self.trajectories[pid]["beta"][-1][0]
 
-                        if not numpy.allclose(current - [alpha, beta], 0, atol=0.1):
-                            really_not_moving.append(pid)
+            #             if not numpy.allclose(current - [alpha, beta], 0, atol=0.1):
+            #                 really_not_moving.append(pid)
 
-                    if len(really_not_moving) > 0:
-                        not_moving_str = ", ".join(map(str, really_not_moving))
-                        # Should this be an error?
-                        warnings.warn(
-                            "Some positioners do not appear to be "
-                            f"moving: {not_moving_str}.",
-                            JaegerUserWarning,
-                        )
+            #         if len(really_not_moving) > 0:
+            #             not_moving_str = ", ".join(map(str, really_not_moving))
+            #             # Should this be an error?
+            #             warnings.warn(
+            #                 "Some positioners do not appear to be "
+            #                 f"moving: {not_moving_str}.",
+            #                 JaegerUserWarning,
+            #             )
 
             while True:
 
@@ -600,14 +689,41 @@ class Trajectory(object):
             raise
 
         finally:
-            await self.fps.stop_trajectory()
-            await self.fps.update_position()
+            # Not explicitely updating the positions here because save_snapshot()
+            # will do that and no need to waste extra time.
+            await self.fps.save_snapshot()
+
+            if self.dump_file:
+                self.dump_trajectory()
+
             self.end_time = time.time()
             if restart_pollers:
                 self.fps.pollers.start()
 
         return True
 
+    def dump_trajectory(self, path: str | None = None):
+        """Dumps the trajectory to a JSON file."""
+
+        if path is None:
+            if self.dump_file is None:
+                return
+
+            path = self.dump_file
+
+        self.dump_data["success"] = not self.failed
+        self.dump_data["trajectory_start_time"] = self.start_time
+        self.dump_data["trajectory_send_time"] = self.data_send_time
+        self.dump_data["use_sync_line"] = self.use_sync_line
+        self.dump_data["end_time"] = time.time()
+        self.dump_data["final_positions"] = self.fps.get_positions_dict()
+
+        if not os.path.exists(os.path.dirname(path)):
+            os.makedirs(os.path.dirname(path))
+
+        with open(path, "w") as f:
+            f.write(json.dumps(self.dump_data, indent=2))
+
     async def abort_trajectory(self):
         """Aborts the trajectory transmission."""
 
