@@ -35,7 +35,9 @@ from zc.lockfile import LockFile
 
 import jaeger
 from jaeger import can_log, config, log, start_file_loggers
+from jaeger.alerts import AlertsBot
 from jaeger.can import JaegerCAN
+from jaeger.chiller import ChillerBot
 from jaeger.commands import (
     Command,
     CommandID,
@@ -93,7 +95,7 @@ class BaseFPS(Dict[int, Positioner], Generic[FPS_T]):
     """
 
     positioner_class: ClassVar[Type[Positioner]] = Positioner
-    _instance: ClassVar[dict[Type[BaseFPS], FPS_T]] = {}
+    _instance: dict[Type[BaseFPS], FPS_T] = {}
 
     initialised: bool
 
@@ -213,7 +215,6 @@ class FPS(BaseFPS["FPS"]):
 
     can: JaegerCAN | str | None = None
     ieb: Union[bool, IEB, dict, str, pathlib.Path, None] = None
-    configuration: BaseConfiguration | None = None
     status: FPSStatus = FPSStatus.IDLE | FPSStatus.TEMPERATURE_NORMAL
 
     def __post_init__(self):
@@ -268,6 +269,15 @@ class FPS(BaseFPS["FPS"]):
         self.__status_event = asyncio.Event()
         self.__temperature_task: asyncio.Task | None = None
 
+        self.observatory = config["observatory"]
+
+        self.alerts = AlertsBot(self)
+        self.chiller = ChillerBot(self)
+
+        self._configuration: BaseConfiguration | None = None
+        self._previous_configurations: list[BaseConfiguration] = []
+        self._preloaded_configuration: BaseConfiguration | None = None
+
         # Position and status pollers
         self.pollers = PollerList(
             [
@@ -294,7 +304,6 @@ class FPS(BaseFPS["FPS"]):
         initialise=True,
         start_pollers: bool | None = None,
         enable_low_temperature: bool = True,
-        use_lock: bool = True,
     ) -> "FPS":
         """Starts the CAN bus and .
 
@@ -304,15 +313,13 @@ class FPS(BaseFPS["FPS"]):
             Whether to initialise the FPS.
         start_pollers
             Whether to initialise the pollers.
-        use_lock
-            Use a lock file to prevent multiple instances of `.FPS`.
         kwargs
             Parameters to pass to `.FPS`.
 
         """
 
         instance = cls(can=can, ieb=ieb)
-        await instance.start_can(use_lock=use_lock)
+        await instance.start_can()
 
         if initialise:
             await instance.initialise(
@@ -322,9 +329,11 @@ class FPS(BaseFPS["FPS"]):
 
         return instance
 
-    async def start_can(self, use_lock: bool = True):
+    async def start_can(self):
         """Starts the JaegerCAN interface."""
 
+        use_lock = config["fps"]["use_lock"]
+
         if use_lock and self.pid_lock is None:
             try:
                 if not os.path.exists(os.path.dirname(LOCK_FILE)):
@@ -338,11 +347,8 @@ class FPS(BaseFPS["FPS"]):
                 )
 
         if isinstance(self.can, JaegerCAN):
-            if self.can._started:
-                return
-            else:
-                await self.can.start()
-                return
+            await self.can.start()
+            return
 
         self.can = await JaegerCAN.create(self.can, fps=self)
         return True
@@ -368,11 +374,29 @@ class FPS(BaseFPS["FPS"]):
 
         return positioner
 
+    @property
+    def configuration(self):
+        """Returns the configuration."""
+
+        return self._configuration
+
+    @configuration.setter
+    def configuration(self, new: BaseConfiguration | None):
+        """Sets the new configuration."""
+
+        # Store current configuration.
+        if self._configuration is not None:
+            self._previous_configurations.append(self._configuration)
+
+        # Keep only 10 previous configurations.
+        self._previous_configurations = self._previous_configurations[-10:]
+
+        self._configuration = new
+
     async def initialise(
         self: T,
         start_pollers: bool | None = None,
         enable_low_temperature: bool = True,
-        use_lock: bool = True,
     ) -> T:
         """Initialises all positioners with status and firmware version.
 
@@ -396,7 +420,7 @@ class FPS(BaseFPS["FPS"]):
             await self.pollers.stop()
 
         # Make sure CAN buses are connected.
-        await self.start_can(use_lock=use_lock)
+        await self.start_can()
 
         # Test IEB connection.
         if isinstance(self.ieb, IEB):
@@ -432,14 +456,9 @@ class FPS(BaseFPS["FPS"]):
 
         # Loops over each reply and set the positioner status to OK. If the
         # positioner was not in the list, adds it.
-        ignored_positioners = []
         for reply in get_fw_command.replies:
             if reply.positioner_id not in self.positioners:
 
-                if reply.positioner_id in config["fps"]["skip_positioners"]:
-                    ignored_positioners.append(reply.positioner_id)
-                    continue
-
                 if hasattr(reply.message, "interface"):
                     interface = reply.message.interface
                     bus = reply.message.bus
@@ -455,12 +474,20 @@ class FPS(BaseFPS["FPS"]):
             if positioner.positioner_id in config["fps"]["disabled_positioners"]:
                 positioner.disabled = True
 
-        if len(ignored_positioners) > 0:
-            warnings.warn(
-                "The following connected positioners are ignored as they are "
-                f"in the skip_positioners list: {ignored_positioners}",
-                JaegerUserWarning,
-            )
+        # Add offline robots. Offline positioners are physically in the array but
+        # they don't reply to commands and we need to specify their position. Once
+        # That's done they behave as normal disabled robots.
+        if config["fps"]["offline_positioners"] is not None:
+            for pid in config["fps"]["offline_positioners"]:
+                off_alpha, off_beta = config["fps"]["offline_positioners"][pid]
+                if pid not in self.positioners:
+                    positioner = self.add_positioner(pid)
+                else:
+                    positioner = self.positioners[pid]
+                positioner.disabled = True
+                positioner.offline = True
+                positioner.alpha = off_alpha
+                positioner.beta = off_beta
 
         # Mark as initialised here although we have some more work to do.
         self.initialised = True
@@ -581,6 +608,11 @@ class FPS(BaseFPS["FPS"]):
         if start_pollers and not self.is_bootloader():
             self.pollers.start()
 
+        # Initialise alerts and chiller bots with a bit of delay to let the actor
+        # time to start.
+        asyncio.create_task(self.alerts.start(delay=5))
+        asyncio.create_task(self.chiller.start(delay=5))
+
         return self
 
     def set_status(self, status: FPSStatus):
@@ -781,19 +813,21 @@ class FPS(BaseFPS["FPS"]):
         await self.update_status()
 
         if jaeger.actor_instance:
-            jaeger.actor_instance.write("e", {"locked": True})
+            jaeger.actor_instance.write(
+                "e",
+                {"locked": True, "locked_by": self.locked_by},
+            )
 
         if snapshot:
-            if by is not None and len(by) > 0:
-                highlight = by[0]
+            if self.locked_by is not None:
+                highlight = self.locked_by[0]
             else:
                 highlight = None
+
+            log.debug(f"Saving snapshot with highlight {highlight} ({self.locked_by})")
             filename = await self.save_snapshot(highlight=highlight)
             warnings.warn(f"Snapshot for locked FPS: {filename}", JaegerUserWarning)
 
-            if jaeger.actor_instance:
-                jaeger.actor_instance.write("i", {"snapshot": filename})
-
     async def unlock(self, force=False):
         """Unlocks the `.FPS` if all collisions have been resolved."""
 
@@ -826,6 +860,18 @@ class FPS(BaseFPS["FPS"]):
 
         return numpy.array(data)
 
+    def get_positions_dict(
+        self,
+        ignore_disabled=False,
+    ) -> dict[int, tuple[float | None, float | None]]:
+        """Returns the alpha and beta positions as a dictionary."""
+
+        return {
+            p.positioner_id: (p.alpha, p.beta)
+            for p in self.positioners.values()
+            if ignore_disabled is False or p.disabled is False
+        }
+
     async def update_status(
         self,
         positioner_ids: Optional[int | List[int]] = None,
@@ -1006,6 +1052,20 @@ class FPS(BaseFPS["FPS"]):
 
         return True
 
+    async def is_folded(self):
+        """Returns `True` if the array if folded."""
+
+        alphaL, betaL = config["kaiju"]["lattice_position"]
+
+        await self.update_position()
+        positions_array = self.get_positions(ignore_disabled=True)
+
+        if len(positions_array) == 0:
+            return False
+
+        lattice: Any = numpy.array([alphaL, betaL])
+        return numpy.allclose(positions_array[:, 1:] - lattice, 0, atol=1)
+
     async def stop_trajectory(self, clear_flags=False):
         """Stops all the positioners without clearing collided flags.
 
@@ -1042,58 +1102,41 @@ class FPS(BaseFPS["FPS"]):
 
     async def goto(
         self,
-        positioner_ids: int | List[int] | None,
-        alpha: float | list | numpy.ndarray,
-        beta: float | list | numpy.ndarray,
-        speed: Optional[Tuple[float, float]] = None,
+        new_positions: dict[int, tuple[float, float]],
+        speed: Optional[float] = None,
         relative=False,
-        force: bool = False,
         use_sync_line: bool | None = None,
+        go_cowboy: bool = False,
     ):
         """Sends a list of positioners to a given position.
 
         Parameters
         ----------
-        positioner_ids
-            The list of positioner_ids to command. If `None`, uses all conencted
-            positioners.
-        alpha
-            The alpha angle. Can be an array with the same size of the list of
-            positioner IDs. Otherwise sends all the positioners to the same angle.
-        beta
-            The beta angle.
+        new_positions
+            The new positions as a dictionary of positioner ID to a tuple of new
+            alpha and beta angles. Positioners not specified will be kept on the
+            same positions.
         speed
-            As a tuple, the alpha and beta speeds to use. If `None`, uses the default
-            ones.
+            The speed to use.
         relative
             If `True`, ``alpha`` and ``beta`` are considered relative angles.
-        force
-            If ``positioners_ids=None``, ``force`` must be set to `True` to move
-            the entire array.
         use_sync_line
             Whether to use the SYNC line to start the trajectories.
+        go_cowboy
+            If set, does not create a ``kaiju``-safe trajectory. Use at your own risk.
 
         """
 
-        if isinstance(positioner_ids, int):
-            positioner_ids = [positioner_ids]
-
-        if (positioner_ids is None or len(positioner_ids) == 0) and force is False:
-            raise JaegerError("Moving all positioners requires force=True.")
-
-        positioner_ids = positioner_ids or list(self.keys())
-
         try:
             traj = await goto(
                 self,
-                positioner_ids,
-                alpha,
-                beta,
+                new_positions,
                 relative=relative,
                 speed=speed,
                 use_sync_line=use_sync_line,
+                go_cowboy=go_cowboy,
             )
-        except JaegerError:
+        except Exception:
             raise
         finally:
             await self.update_status()
@@ -1173,8 +1216,8 @@ class FPS(BaseFPS["FPS"]):
         self,
         path: Optional[str | pathlib.Path] = None,
         collision_buffer: float | None = None,
-        return_axes: bool = False,
         highlight: int | None = None,
+        write_to_actor: bool = True,
     ) -> str | Axes:
         """Creates a plot with the current arrangement of the FPS array.
 
@@ -1182,46 +1225,52 @@ class FPS(BaseFPS["FPS"]):
         ----------
         path
             The path where to save the plot. Defaults to
-            ``/data/fps/snapshots/MJD/fps_snapshot_<SEQ>.pdf``.
+            ``/data/logs/jaeger/snapshots/MJD/fps_snapshot_<SEQ>.pdf``.
         collision_buffer
             The collision buffer.
-        return_axes
-            If `True`, returns the matplotlib axes instead of saving the plot.
+        highlight
+            A robot ID to highlight.
 
         """
 
         from jaeger.kaiju import get_snapshot
 
-        ax = await get_snapshot(collision_buffer=collision_buffer, highlight=highlight)
-
-        if return_axes is True:
-            return ax
-
         if path is not None:
-            ax.figure.savefig(path)
-            return str(path)
+            path = str(path)
 
-        mjd = int(Time.now().mjd)
-        dirpath = f"/data/fps/snapshots/{mjd}"
-        if not os.path.exists(dirpath):
-            os.makedirs(dirpath)
-
-        path_pattern = dirpath + "/fps_snapshot_*.pdf"
-        files = sorted(glob(path_pattern))
-
-        if len(files) == 0:
-            seq = 1
         else:
-            seq = int(files[-1].split("_")[-1][0:4]) + 1
+            mjd = int(Time.now().mjd)
+            dirpath = os.path.join(config["fps"]["snapshot_path"], str(mjd))
+            if not os.path.exists(dirpath):
+                os.makedirs(dirpath)
 
-        path = path_pattern.replace("*", f"{seq:04d}")
-        ax.figure.savefig(path)
+            path_pattern = dirpath + "/fps_snapshot_*.pdf"
+            files = sorted(glob(path_pattern))
+
+            if len(files) == 0:
+                seq = 1
+            else:
+                seq = int(files[-1].split("_")[-1][0:4]) + 1
+
+            path = path_pattern.replace("*", f"{mjd}_{seq:04d}")
+
+        result = await get_snapshot(
+            path,
+            collision_buffer=collision_buffer,
+            highlight=highlight,
+        )
+
+        if result is True and write_to_actor is True and jaeger.actor_instance:
+            jaeger.actor_instance.write("i", {"snapshot": path})
 
         return path
 
     async def _handle_temperature(self):
         """Handle positioners in low temperature."""
 
+        if not isinstance(self.ieb, IEB):
+            log.error("Cannot handle low-temperature mode. IEB not present.")
+
         async def set_rpm(activate):
             if activate:
                 rpm = config["low_temperature"]["rpm_cold"]
@@ -1292,15 +1341,15 @@ class FPS(BaseFPS["FPS"]):
                     self.set_status(base_status | FPSStatus.TEMPERATURE_NORMAL)
 
             except BaseException as err:
-                log.info(
+                log.warning(
                     f"Cannot read device {sensor!r}. "
-                    f"Low-temperature mode will not be engaged: {err}",
+                    f"Low-temperature tracking temporarily disabled: {err}",
                 )
                 base_status = self.status & ~FPSStatus.TEMPERATURE_BITS
                 self.set_status(base_status | FPSStatus.TEMPERATURE_UNKNOWN)
-                return
 
-            await asyncio.sleep(interval)
+            finally:
+                await asyncio.sleep(interval)
 
     async def shutdown(self):
         """Stops pollers and shuts down all remaining tasks."""
@@ -1313,6 +1362,9 @@ class FPS(BaseFPS["FPS"]):
         if self.pollers:
             await self.pollers.stop()
 
+        await self.chiller.stop()
+        await self.alerts.stop()
+
         log.debug("Cancelling all pending tasks and shutting down.")
 
         tasks = [
@@ -1334,3 +1386,9 @@ class FPS(BaseFPS["FPS"]):
 
     async def __aexit__(self, *excinfo):
         await self.shutdown()
+
+    def __copy__(self):
+        return self
+
+    def __deepcopy__(self, memo):
+        return self
