@@ -17,15 +17,13 @@ from typing import TYPE_CHECKING, Optional
 
 import numpy
 import pandas
-import sep
 from astropy.io import fits
 from astropy.table import Table
-from matplotlib import pyplot as plt
 
 from clu.command import Command
 from clu.legacy.tron import TronConnection
 from coordio.defaults import calibration
-from coordio.transforms import RoughTransform, ZhaoBurgeTransform
+from coordio.transforms import FVCTransformAPO
 
 from jaeger import config, log
 from jaeger.exceptions import FVCError, JaegerUserWarning, TrajectoryError
@@ -39,17 +37,23 @@ from jaeger.utils import run_in_executor
 if TYPE_CHECKING:
     from jaeger.actor import JaegerActor
 
+
 __all__ = ["FVC"]
 
 
+FVC_CONFIG = config["fvc"]
+
+
 class FVC:
     """Focal View Camera class."""
 
     fibre_data: Optional[pandas.DataFrame]
+    measurements: Optional[pandas.DataFrame]
     centroids: Optional[pandas.DataFrame]
     offsets: Optional[pandas.DataFrame]
 
     image_path: Optional[str]
+    proc_image_path: Optional[str]
     raw_hdu: Optional[fits.ImageHDU]
     proc_hdu: Optional[fits.ImageHDU]
 
@@ -65,6 +69,7 @@ class FVC:
             )
 
         self.site = site
+        self.correction_applied: bool = False
 
         self.command = command
         self.fps = FPS.get_instance()
@@ -74,16 +79,20 @@ class FVC:
     def reset(self):
         """Resets the instance."""
 
+        self.fvc_transform: FVCTransformAPO | None = None
         self.fibre_data = None
         self.centroids = None
         self.offsets = None
 
         self.image_path = None
+        self.proc_image_path = None
         self.raw_hdu = None
         self.proc_hdu = None
 
         self.k = 1
-        self.fitrms = -999.0
+        self.fitrms = -9.99
+        self.perc_90 = -9.99
+        self.fvc_percent_reached = -9.99
 
     def set_command(self, command: Command[JaegerActor]):
         """Sets the command."""
@@ -139,7 +148,10 @@ class FVC:
             if self.command.status.is_done:
                 raise FVCError("Command is done.")
 
-        self.log(f"Taking {exposure_time} seconds FVC exposure.", to_command=False)
+        self.log(
+            f"Taking {stack} x {exposure_time} seconds FVC exposure.",
+            to_command=False,
+        )
 
         tron = None
 
@@ -152,6 +164,7 @@ class FVC:
             expose_command = self.command.send_command("fliswarm", cmd_str)
         else:
             tron = TronConnection(
+                "jaeger.jaeger",
                 config["actor"]["tron_host"],
                 config["actor"]["tron_port"],
             )
@@ -167,22 +180,23 @@ class FVC:
         if expose_command.status.did_fail:
             raise FVCError("The FVC exposure failed.")
 
-        for reply in expose_command.replies:
-            for keyword in reply.keywords:
-                if keyword.name.lower() == "filename":
-                    filename = keyword.values[-1]
-                    self.log(f"FVC raw image is {filename}.", to_command=False)
-                    return pathlib.Path(filename)
-
-        raise FVCError("The exposure succeeded but did not output the filename.")
+        try:
+            filename = expose_command.replies.get("filename")[-1]
+            self.log(f"FVC raw image is {filename}.", to_command=False)
+            return pathlib.Path(filename)
+        except KeyError:
+            raise FVCError("The exposure succeeded but did not output the filename.")
 
     def process_fvc_image(
         self,
         path: pathlib.Path | str,
+        positioner_coords: dict,
         fibre_data: Optional[pandas.DataFrame] = None,
         fibre_type: str = "Metrology",
+        use_winpos: bool = True,
+        use_new_invkin: bool = True,
         plot: bool | str = False,
-        polids: numpy.ndarray | list | None = None,
+        outdir: str | None = None,
     ) -> tuple[fits.ImageHDU, pandas.DataFrame, pandas.DataFrame]:
         """Processes a raw FVC image.
 
@@ -190,6 +204,9 @@ class FVC:
         ----------
         path
             The path to the raw FVC image.
+        positioner_coords
+            A dictionary of positioner ID to ``(alpha, beta)`` with the current
+            positions of the robots.
         fibre_data
             A Pandas data frame with the expected coordinates of the targets. It
             is expected the data frame will have columns ``positioner_id``,
@@ -200,6 +217,10 @@ class FVC:
         fibre_type
             The ``fibre_type`` rows in ``fibre_data`` to use. Defaults to
             ``fibre_type='Metrology'``.
+        use_winpos
+            Whether to use windowed position for centroid extraction.
+        use_new_invkin
+            Use new inverse kinnematic to calculate alpha/beta.
         plot
             Whether to save additional debugging plots along with the processed image.
             If ``plot`` is a string, it will be used as the directory to which to
@@ -227,11 +248,13 @@ class FVC:
                 raise FVCError("No fibre data and no configuration has been loaded.")
             fibre_data = self.fps.configuration.assignment_data.fibre_table
 
-        self.fibre_data = fibre_data.copy().reset_index().set_index("fibre_type")
+        fdata = fibre_data.copy().reset_index().set_index("positioner_id")
 
         self.log(f"Processing raw image {path}")
 
         dirname, base = os.path.split(path)
+        dirname = outdir or dirname
+
         proc_path_root = os.path.join(dirname, "proc-" + base[0 : base.find(".fit")])
 
         if plot is True:
@@ -252,149 +275,109 @@ class FVC:
 
         self.log(f"Max counts in image: {numpy.max(image_data)}", level=logging.INFO)
 
-        self.centroids = self.extract(image_data)
+        # Get the rotator angle so that we can derotate the centroids to the
+        # x/ywok-aligned configuration.
+        rotpos = hdus[1].header.get("IPA", None)
+        if rotpos is None:
+            raise FVCError("IPA keyword not found in the header.")
+        rotpos = float(rotpos) % 360.0
 
-        fiducialCoords = calibration.fiducialCoords.loc[self.site]
+        positioner_df = pandas.DataFrame.from_dict(
+            positioner_coords,
+            columns=["alphaReport", "betaReport"],
+            orient="index",
+        )
+        positioner_df.index.set_names(["positionerID"], inplace=True)
 
-        xCMM = fiducialCoords.xWok.to_numpy()
-        yCMM = fiducialCoords.yWok.to_numpy()
-        xyCMM = numpy.array([xCMM, yCMM]).T
-
-        xyCCD = self.centroids[["x", "y"]].to_numpy()
-
-        fibre_data_met = self.fibre_data.loc[fibre_type]
-
-        # Get close enough to associate the correct centroid with the correct fiducial.
-        x_wok_expect = numpy.concatenate([xCMM, fibre_data_met.xwok.to_numpy()])
-        y_wok_expect = numpy.concatenate([yCMM, fibre_data_met.ywok.to_numpy()])
-        xy_wok_expect = numpy.array([x_wok_expect, y_wok_expect]).T
-
-        rt = RoughTransform(xyCCD, xy_wok_expect)
-        xy_wok_rough = rt.apply(xyCCD)
-
-        # First associate fiducials and build first round just use outer fiducials
-        rCMM = numpy.sqrt(xyCMM[:, 0] ** 2 + xyCMM[:, 1] ** 2)
-        keep = rCMM > 310
-        xyCMMouter = xyCMM[keep, :]
-
-        arg_found, fid_rough_dist = arg_nearest_neighbor(xyCMMouter, xy_wok_rough)
-        self.log(
-            f"Max fiducial rough distance: {numpy.max(fid_rough_dist):.3f}",
-            level=logging.INFO,
+        fvc_transform = FVCTransformAPO(
+            image_data,
+            positioner_df,
+            rotpos,
+            plotPathPrefix=plot_path_root,
         )
 
-        xy_fiducial_CCD = xyCCD[arg_found]
-        xy_fiducial_wok_rough = xy_wok_rough[arg_found]
+        self.centroids = fvc_transform.extractCentroids()
+        fvc_transform.fit(useWinpos=use_winpos, newInvKin=use_new_invkin)
 
-        if plot:
-            self.plot_fvc_assignments(
-                xy_wok_rough,
-                fibre_data_met,
-                xCMM,
-                yCMM,
-                plot_path_root + "_roughassoc.pdf",
-                xy_fiducial=xy_fiducial_wok_rough,
-                xy_fiducial_cmm=xyCMMouter,
-                title="Rough fiducial association",
+        assert fvc_transform.positionerTableMeas is not None
+
+        measured = fvc_transform.positionerTableMeas.copy().set_index("positionerID")
+
+        n_dubious = measured.wokErrWarn.sum()
+        if n_dubious > 0:
+            self.log(
+                f"Found {n_dubious} positioners with dubious centroid matches.",
+                level=logging.WARNING,
             )
 
-        ft = ZhaoBurgeTransform(
-            xy_fiducial_CCD,
-            xyCMMouter,
-            polids=(polids or config["fvc"]["zb_polids"]),
-        )
-        self.log(
-            f"Full transform 1. Bisased RMS={ft.rms * 1000:.3f}, "
-            f"Unbiased RMS={ft.unbiasedRMS * 1000:.3f}.",
-            level=logging.INFO,
-        )
-        xy_wok_meas = ft.apply(xyCCD, zb=False)
+        metrology_data = fdata.copy().reset_index()
+        metrology_data = metrology_data.loc[metrology_data.fibre_type == fibre_type]
 
-        if plot:
-            self.plot_fvc_assignments(
-                xy_wok_meas,
-                fibre_data_met,
-                xCMM,
-                yCMM,
-                plot_path_root + "_full1.pdf",
-                title="Full transform 1",
-            )
+        # Create a column to mark positioners with dubious matches.
+        fdata.loc[:, "dubious"] = 0
+        dubious_pid = measured.loc[measured.wokErrWarn].index.values
+        fdata.loc[dubious_pid, "dubious"] = 1
 
-        # Re-associate fiducials, some could have been wrongly associated in
-        # first fit but second fit should be better?
-        arg_found, fid_rough_dist = arg_nearest_neighbor(xyCMM, xy_wok_meas)
-        self.log(
-            f"Max fiducial fit 2 distance: {numpy.max(fid_rough_dist):.3f}",
-            level=logging.INFO,
-        )
-
-        xy_fiducial_CCD = xyCCD[arg_found]  # Overwriting
-        xy_fiducial_wok_refine = xy_wok_meas[arg_found]
-
-        if plot:
-            self.plot_fvc_assignments(
-                xy_wok_meas,
-                fibre_data_met,
-                xCMM,
-                yCMM,
-                plot_path_root + "_refineassoc.pdf",
-                title="Refined fiducial association",
-                xy_fiducial=xy_fiducial_wok_refine,
-                xy_fiducial_cmm=xyCMM,
-            )
-
-        # Try a new transform
-        ft = ZhaoBurgeTransform(
-            xy_fiducial_CCD,
-            xyCMM,
-            polids=(polids or config["fvc"]["zb_polids"]),
-        )
-        self.log(
-            f"Full transform 2. Bisased RMS={ft.rms * 1000:.3f}, "
-            f"Unbiased RMS={ft.unbiasedRMS * 1000:.3f}.",
-            level=logging.INFO,
-        )
-
-        xy_wok_meas = ft.apply(xyCCD)  # Overwrite
-
-        if plot:
-            self.plot_fvc_assignments(
-                xy_wok_meas,
-                fibre_data_met,
-                xCMM,
-                yCMM,
-                plot_path_root + "_full2.pdf",
-                title="Full transform 2",
-            )
-
-        # Transform all CCD detections to wok space
-        xy_expect_pos = fibre_data_met[["xwok", "ywok"]].to_numpy()
-
-        arg_found, met_dist = arg_nearest_neighbor(xy_expect_pos, xy_wok_meas)
-        self.log(
-            f"Max metrology distance: {numpy.max(met_dist):.3f}",
-            level=logging.INFO,
-        )
-        xy_wok_robot_meas = xy_wok_meas[arg_found]
-
-        self.fibre_data.loc[fibre_type, "xwok_measured"] = xy_wok_robot_meas[:, 0]
-        self.fibre_data.loc[fibre_type, "ywok_measured"] = xy_wok_robot_meas[:, 1]
-
-        # Only use online robots for final RMS.
-        online = self.fibre_data.loc[
-            (self.fibre_data.index == fibre_type) & (self.fibre_data.offline == 0)
+        wok_measured = measured.loc[
+            metrology_data.positioner_id, ["xWokMeasMetrology", "yWokMeasMetrology"]
         ]
-        dx = online.xwok - online.xwok_measured
-        dy = online.ywok - online.ywok_measured
+        fdata.loc[
+            fdata.fibre_type == fibre_type, ["xwok_measured", "ywok_measured"]
+        ] = wok_measured.values
 
-        self.fitrms = numpy.sqrt(numpy.mean(dx ** 2 + dy ** 2))
+        fdata = fdata.reset_index().set_index("fibre_type")
+
+        # Only use online, assigned robots for final RMS. First get groups of fibres
+        # with an assigned robot, that are not offline or dubious.
+        if fdata.assigned.sum() > 0:
+            assigned = fdata.groupby("positioner_id").filter(
+                lambda g: g.assigned.any()
+                & (g.offline == 0).all()
+                & (g.dubious == 0).all()
+                # & (g.on_target == 1).any()
+            )
+        else:
+            self.log("No assigned fibres found. Using all matched fibres.")
+            assigned = fdata.groupby("positioner_id").filter(
+                lambda g: (g.offline == 0).all()
+                & (g.dubious == 0).all()
+                # & (g.on_target == 1).any()
+            )
+
+        # Now get the metrology fibre from those groups.
+        assigned = assigned.loc[fibre_type]
+
+        # Calculate RMS from assigned fibres.
+        dx = assigned.xwok - assigned.xwok_measured
+        dy = assigned.ywok - assigned.ywok_measured
+        self.fitrms = numpy.round(numpy.sqrt(numpy.mean(dx**2 + dy**2)), 5)
         self.log(f"RMS full fit {self.fitrms * 1000:.3f} um.")
 
-        hdus[1].header["FITRMS"] = (self.fitrms * 1000, "RMS full fit [um]")
+        # Also calculate 90% percentile and percentage of targets below threshold.
+        distance = numpy.sqrt(dx**2 + dy**2)
 
-        self.fibre_data.reset_index(inplace=True)
-        self.fibre_data.set_index(["hole_id", "fibre_type"], inplace=True)
+        self.perc_90 = numpy.round(numpy.percentile(distance, 90), 4)
+
+        n_reached = numpy.sum(distance <= (config["fvc"]["target_distance"] / 1000))
+        self.fvc_percent_reached = numpy.round(n_reached / len(dx) * 100, 1)
+
+        # FITSRMS is the RMS of measured - expected for assigned, non-disabled
+        # robots. This is different from FVC_RMS reported by
+        # FVCTransformAPO.getMetadata() that is measured - reported for all
+        # positioners.
+        hdus[1].header["FITRMS"] = (self.fitrms * 1000, "RMS full fit [um]")
+        hdus[1].header["PERC90"] = (self.perc_90 * 1000, "90% percentile [um]")
+        hdus[1].header["FVCREACH"] = (
+            self.fvc_percent_reached,
+            "Targets that have reached their goal [%]",
+        )
+
+        fdata.reset_index(inplace=True)
+        fdata.set_index(["hole_id", "fibre_type"], inplace=True)
+
+        self.fibre_data = fdata
         self.proc_hdu = hdus[1]
+        self.fvc_transform = fvc_transform
 
         self.log(f"Finished processing {path}", level=logging.DEBUG)
 
@@ -405,6 +388,7 @@ class FVC:
         reported_positions: numpy.ndarray,
         fibre_data: Optional[pandas.DataFrame] = None,
         k: Optional[float] = None,
+        max_correction: Optional[float] = None,
     ) -> pandas.DataFrame:
         """Determines the offset to apply to the currently reported positions.
 
@@ -426,6 +410,9 @@ class FVC:
             last run.
         k
             The fraction of the correction to apply.
+        max_correction
+            The maximum offset allowed per robot and arm, in degrees. Corrections
+            larger than ``max_offset`` are clipped.
 
         Returns
         -------
@@ -435,10 +422,12 @@ class FVC:
 
         """
 
-        self.log("Calculating offset from FVC image and fit.")
+        self.log("Calculating offset from FVC image and model fit.")
 
         site = config["observatory"]
+
         self.k = k or config["fvc"]["k"]
+        max_offset: float = max_correction or config["fvc"]["max_correction"]
 
         if fibre_data is None and self.fibre_data is None:
             raise FVCError("No fibre data passed or stored in the instance.")
@@ -535,7 +524,7 @@ class FVC:
         pos_na = offsets["alpha_measured"].isna()
         if pos_na.sum() > 0:
             self.log(
-                "Failed to calculated corrected positioner coordinates for "
+                "Failed to calculate corrected positioner coordinates for "
                 f"{pos_na.sum()} positioners.",
                 level=logging.WARNING,
             )
@@ -554,7 +543,6 @@ class FVC:
         offsets["beta_offset"] = beta_offset
 
         # Clip very large offsets and apply a proportional term.
-        max_offset = config["fvc"]["max_offset"]
         alpha_offset_c = numpy.clip(self.k * alpha_offset, -max_offset, max_offset)
         beta_offset_c = numpy.clip(self.k * beta_offset, -max_offset, max_offset)
 
@@ -642,6 +630,13 @@ class FVC:
 
         proc_hdus = fits.HDUList([fits.PrimaryHDU(), self.proc_hdu])
 
+        if self.fps and self.fps.configuration:
+            proc_hdus[1].header["CONFIGID"] = self.fps.configuration.configuration_id
+        else:
+            proc_hdus[1].header["CONFIGID"] = -999.0
+
+        proc_hdus[1].header["CAPPLIED"] = self.correction_applied
+
         positionerTable = calibration.positionerTable
         wokCoords = calibration.wokCoords
         fiducialCoords = calibration.fiducialCoords
@@ -657,11 +652,11 @@ class FVC:
             table = fits.BinTableHDU(rec, name=name)
             proc_hdus.append(table)
 
-        measured_coords = self.fibre_data.copy()
-        measured_coords.reset_index(inplace=True)
-        measured_coords.sort_values("positioner_id", inplace=True)
-        measured_coords_rec = Table.from_pandas(measured_coords).as_array()
-        proc_hdus.append(fits.BinTableHDU(measured_coords_rec, name="FIBERDATA"))
+        fibre_data = self.fibre_data.copy()
+        fibre_data.reset_index(inplace=True)
+        fibre_data.sort_values("positioner_id", inplace=True)
+        fibre_data_rec = Table.from_pandas(fibre_data).as_array()
+        proc_hdus.append(fits.BinTableHDU(fibre_data_rec, name="FIBERDATA"))
 
         # Add IEB information
         ieb_keys = config["fvc"]["ieb_keys"]
@@ -680,6 +675,11 @@ class FVC:
         for key, val in ieb_data.items():
             proc_hdus[1].header[key] = val
 
+        # Add header keywords from coordio.FVCTransfromAPO.
+        if self.fvc_transform:
+            proc_hdus[1].header.extend(self.fvc_transform.getMetadata())
+
+        posangles = None
         if self.fps:
             await self.fps.update_position()
             positions = self.fps.get_positions()
@@ -717,19 +717,21 @@ class FVC:
                     current_positions["startAlpha"] = _start_alpha
                     current_positions["startBeta"] = _start_beta
 
-            rec = Table.from_pandas(current_positions.reset_index()).as_array()
-            proc_hdus.append(fits.BinTableHDU(rec, name="POSANGLES"))
+            posangles = Table.from_pandas(current_positions).as_array()
+        proc_hdus.append(fits.BinTableHDU(posangles, name="POSANGLES"))
 
-        rec = Table.from_pandas(self.centroids.reset_index()).as_array()
+        rec = Table.from_pandas(self.centroids).as_array()
         proc_hdus.append(fits.BinTableHDU(rec, name="CENTROIDS"))
 
+        offsets = None
         if self.offsets is not None:
-            rec = Table.from_pandas(self.offsets.reset_index()).as_array()
-            proc_hdus.append(fits.BinTableHDU(rec, name="OFFSETS"))
+            offsets = Table.from_pandas(self.offsets.reset_index()).as_array()
+        proc_hdus.append(fits.BinTableHDU(offsets, name="OFFSETS"))
 
         await run_in_executor(proc_hdus.writeto, new_filename, checksum=True)
 
         self.log(f"Processed HDU written to {new_filename}")
+        self.proc_image_path = os.path.abspath(new_filename)
 
         return proc_hdus
 
@@ -739,6 +741,9 @@ class FVC:
     ):  # pragma: no cover
         """Applies the offsets. Fails if the trajectory is collided or deadlock."""
 
+        if self.fps.locked:
+            raise FVCError("The FPS is locked. Cannot apply corrections.")
+
         self.log("Preparing correction trajectory.")
 
         if self.offsets is None and offsets is None:
@@ -750,29 +755,45 @@ class FVC:
         assert offsets is not None
         await self.fps.update_position()
 
+        target_distance = config["fvc"]["target_distance"]
+
         # Setup robot grid.
-        grid = get_robot_grid()
+        grid = get_robot_grid(self.fps)
         for robot in grid.robotDict.values():
+            if robot.isOffline:
+                continue
+
             positioner = self.fps[robot.id]
             robot.setAlphaBeta(positioner.alpha, positioner.beta)
+            robot.setDestinationAlphaBeta(positioner.alpha, positioner.beta)
 
-            row = offsets.loc[robot.id, ["alpha_new", "beta_new"]]
-            if row.isna().any():
-                log.warning(f"Positioner {robot.id}: new position is NaN. Skipping.")
-                robot.setDestinationAlphaBeta(positioner.alpha, positioner.beta)
+            if offsets.loc[robot.id].transformation_valid == 0:
+                continue
+
+            new = offsets.loc[robot.id, ["alpha_new", "beta_new"]]
+            dist = offsets.loc[robot.id, ["xwok_distance", "ywok_distance"]] * 1000.0
+            if numpy.hypot(dist.xwok_distance, dist.ywok_distance) > target_distance:
+                robot.setDestinationAlphaBeta(new.alpha_new, new.beta_new)
             else:
-                robot.setDestinationAlphaBeta(row.alpha_new, row.beta_new)
+                robot.isOffline = True
 
-        # Check for collisions.
-        collided = [rid for rid in grid.robotDict if grid.isCollided(rid)]
+        # Check for collisions. If robots are collided just leave them there.
+        collided = grid.getCollidedRobotList()
         n_coll = len(collided)
         if n_coll > 0:
-            raise FVCError(f"Cannot apply corrections. {n_coll} robots are collided.")
+            for pid in collided:
+                positioner = self.fps[pid]
+                alpha = positioner.alpha
+                beta = positioner.beta
+                grid.robotDict[pid].setAlphaBeta(alpha, beta)
+                grid.robotDict[pid].setDestinationAlphaBeta(alpha, beta)
 
         # Generate trajectories.
         (to_destination, _, did_fail, deadlocks) = await get_path_pair_in_executor(
             grid,
             ignore_did_fail=True,
+            stop_if_deadlock=True,
+            ignore_initial_collisions=True,
         )
         if did_fail:
             log.warning(
@@ -781,130 +802,66 @@ class FVC:
 
         self.log("Sending correction trajectory.")
         try:
-            await self.fps.send_trajectory(to_destination)
+            await self.fps.send_trajectory(to_destination, command=self.command)
         except TrajectoryError as err:
             raise FVCError(f"Failed executing the correction trajectory: {err}")
 
+        self.correction_applied = True
         self.log("Correction applied.")
 
-    def extract(self, image_data: numpy.ndarray) -> pandas.DataFrame:
-        """Extract image data using SExtractor. Returns the extracted centroids."""
+    async def write_summary_F(self):
+        """Updates data with the last measured positions and write confSummaryF."""
 
-        image_data = numpy.array(image_data, dtype=numpy.float32)
+        if self.fps is None or self.fps.configuration is None:
+            raise FVCError("write_summary_F can only be called with a configuration.")
 
-        bkg = sep.Background(image_data)
-        bkg_image = bkg.back()
+        if self.fibre_data is None:
+            raise FVCError("No fibre data.")
 
-        data_sub = image_data - bkg_image
+        self.log("Updating coordinates.", level=logging.DEBUG)
 
-        objects = sep.extract(
-            data_sub,
-            config["fvc"]["background_sigma"],
-            err=bkg.globalrms,
-        )
-        objects = pandas.DataFrame(objects)
-
-        # Eccentricity
-        objects["ecentricity"] = 1 - objects["b"] / objects["a"]
-
-        # Slope of ellipse (optical distortion direction)
-        objects["slope"] = numpy.tan(objects["theta"] + numpy.pi / 2)  # rotate by 90
-
-        # Intercept of optical distortion direction
-        objects["intercept"] = objects["y"] - objects["slope"] * objects["x"]
-
-        # Ignore everything less than X pixels
-        objects = objects.loc[objects["npix"] > config["fvc"]["centroid_min_npix"]]
-
-        self.log(f"Found {len(objects)} centroids", level=logging.INFO)
-
-        ncentroids = len(calibration.positionerTable) + len(calibration.fiducialCoords)
-        self.log(f"Expected {ncentroids} centroids", level=logging.INFO)
-
-        return objects
-
-    def plot_fvc_assignments(
-        self,
-        xy: numpy.ndarray,
-        target_coords: pandas.DataFrame | pandas.Series,
-        xCMM: numpy.ndarray,
-        yCMM: numpy.ndarray,
-        filename: str,
-        xy_fiducial: numpy.ndarray | None = None,
-        xy_fiducial_cmm: numpy.ndarray | None = None,
-        title: str | None = None,
-    ):
-        """Plot the results of the transformation."""
-
-        plt.figure(figsize=(8, 8))
-
-        if title:
-            plt.title(title)
-
-        plt.plot(
-            xy[:, 0],
-            xy[:, 1],
-            "o",
-            ms=4,
-            markerfacecolor="None",
-            markeredgecolor="red",
-            markeredgewidth=1,
-            label="Centroid",
+        fdata = (
+            self.fibre_data.reset_index()
+            .set_index(["positioner_id", "fibre_type"])
+            .copy()
         )
 
-        plt.plot(
-            target_coords.xwok.to_numpy(),
-            target_coords.ywok.to_numpy(),
-            "xk",
-            ms=3,
-            label="Expected MET",
+        idx = pandas.IndexSlice
+        cols = ["hole_id", "xwok_measured", "ywok_measured"]
+        measured = fdata.loc[idx[:, "Metrology"], cols].dropna()
+
+        for (pid, ftype), row in measured.iterrows():
+            (alpha, beta), _ = wok_to_positioner(
+                row.hole_id,
+                self.fps.observatory,
+                "Metrology",
+                row.xwok_measured,
+                row.ywok_measured,
+            )
+            if not numpy.isnan(alpha):
+                fdata.loc[idx[pid, :], ["alpha", "beta"]] = (alpha, beta)
+
+        configuration_copy = self.fps.configuration.copy()
+        configuration_copy.assignment_data.fibre_table = fdata.copy()
+
+        for pid in measured.index.get_level_values(0).tolist():
+            alpha, beta = fdata.loc[(pid, "Metrology"), ["alpha", "beta"]]
+            for ftype in ["APOGEE", "BOSS", "Metrology"]:
+                configuration_copy.assignment_data.positioner_to_icrs(
+                    pid,
+                    ftype,
+                    alpha,
+                    beta,
+                    update=True,
+                )
+
+        await configuration_copy.write_summary(
+            flavour="F",
+            headers={
+                "fvc_rms": self.fitrms,
+                "fvc_90_perc": self.perc_90,
+                "fvc_percent_reached": self.fvc_percent_reached,
+                "fvc_image_path": self.proc_image_path if self.proc_image_path else "",
+            },
+            overwrite=True,
         )
-
-        # Overplot fiducials
-        plt.plot(
-            xCMM,
-            yCMM,
-            "D",
-            ms=6,
-            markerfacecolor="None",
-            markeredgecolor="cornflowerblue",
-            markeredgewidth=1,
-            label="Expected FIF",
-        )
-
-        if xy_fiducial is not None and xy_fiducial_cmm is not None:
-            for cmm, measured in zip(xy_fiducial_cmm, xy_fiducial):
-                plt.plot([cmm[0], measured[0]], [cmm[1], measured[1]], "-k")
-
-        plt.axis("equal")
-        plt.legend()
-        plt.xlim([-350, 350])
-        plt.ylim([-350, 350])
-        plt.savefig(filename, dpi=350)
-        plt.close()
-
-
-def arg_nearest_neighbor(xyA: numpy.ndarray, xyB: numpy.ndarray):
-    """Loop over xy list A, find nearest neighbour in list B
-
-    Returns
-    -------
-    result
-        The indices in list b that match A.
-
-    """
-
-    # TODO: this is probably efficient given the number of points, but maybe
-    # replace with a scipy cdist.
-
-    xyA = numpy.array(xyA)
-    xyB = numpy.array(xyB)
-    out = []
-    distance = []
-    for x, y in xyA:
-        dist = numpy.sqrt((x - xyB[:, 0]) ** 2 + (y - xyB[:, 1]) ** 2)
-        amin = numpy.argmin(dist)
-        distance.append(dist[amin])
-        out.append(amin)
-
-    return numpy.array(out), numpy.array(distance)
