@@ -6,6 +6,7 @@
 # @Filename: ieb.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
+import math
 import os
 
 from typing import Any, Dict
@@ -15,7 +16,37 @@ from drift import Drift, DriftError
 from jaeger import config
 
 
-__all__ = ["IEB", "FVC"]
+__all__ = ["IEB", "FVC", "_get_category_data"]
+
+
+async def _get_category_data(command, category) -> list:
+    """Returns data about a device category."""
+
+    ieb = command.actor.fps.ieb
+    schema = command.actor.model.schema
+
+    items = schema["properties"][category]["items"]
+    measured = []
+
+    async with ieb:
+        for item in items:
+            name = item["title"]
+            type_ = item["type"]
+            device = ieb.get_device(name)
+            value = (await device.read(connect=False))[0]
+            if type_ == "boolean" and device.__type__ == "relay":
+                value = True if value == "closed" else False
+            elif type_ == "integer":
+                value = int(value)
+            elif type_ == "number":
+                if "multipleOf" in item:
+                    precision = int(-math.log10(item["multipleOf"]))
+                else:
+                    precision = 3
+                value = round(value, precision)
+            measured.append(value)
+
+    return measured
 
 
 class IEB(Drift):
@@ -32,6 +63,7 @@ class IEB(Drift):
         super().__init__(*args, **kwargs)
 
         self._categories = None
+        self._n_failures: int = 0
 
     @classmethod
     def create(cls, path=None):
@@ -67,13 +99,23 @@ class IEB(Drift):
         try:
             await Drift.__aenter__(self)
         except DriftError:
-            self.disabled = True
-            raise DriftError("Failed connecting to the IEB. Disabling it.")
+            self._n_failures += 1
+            if self._n_failures >= 5:
+                self.disabled = True
+                raise DriftError("Failed connecting to the IEB. Disabling it.")
+            else:
+                raise DriftError("Failed connecting to the IEB.")
 
     async def __aexit__(self, *args):
 
         await Drift.__aexit__(self, *args)
 
+    def enable(self):
+        """Re-enables the IEB instance."""
+
+        self._n_failures = 0
+        self.disabled = False
+
     async def get_status(self) -> Dict[str, Any]:
         """Returns the status of the IEB components."""
 
@@ -100,3 +142,15 @@ class FVC(IEB):
         default_ieb_path = config["files"]["fvc_config"]
 
         return super().create(default_ieb_path)
+
+
+class Chiller(IEB):
+    """Connects to the chiller Modbus PLC."""
+
+    @classmethod
+    def create(cls, path=None):
+        """Creates a `.Chiller` instance with the default configuration."""
+
+        default_chiller_path = config["files"]["chiller_config"]
+
+        return super().create(default_chiller_path)
